"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Merchant = void 0;
const Constants_1 = require("./Constants");
const PingCompensatedCharacter_1 = require("./PingCompensatedCharacter");
const Tools_1 = require("./Tools");
class Merchant extends PingCompensatedCharacter_1.PingCompensatedCharacter {
    closeMerchantStand() {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [closeMerchantStand].");
        if (!this.stand)
            return Promise.resolve();
        const closed = new Promise((resolve, reject) => {
            const checkStand = (data) => {
                if (!data.stand) {
                    this.socket.removeListener("player", checkStand);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("player", checkStand);
                reject(`closeMerchantStand timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("player", checkStand);
        });
        this.socket.emit("merchant", { close: 1 });
        return closed;
    }
    fish() {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [fish].");
        let startedFishing = false;
        if (this.c.fishing)
            startedFishing = true;
        const fished = new Promise((resolve, reject) => {
            const caughtCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]fishing['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.removeListener("game_response", failCheck1);
                    this.socket.removeListener("ui", failCheck2);
                    this.socket.removeListener("eval", caughtCheck);
                    this.socket.removeListener("player", failCheck3);
                    resolve();
                }
            };
            const failCheck1 = (data) => {
                if (typeof data == "string") {
                    if (data == "skill_cant_wtype") {
                        this.socket.removeListener("game_response", failCheck1);
                        this.socket.removeListener("ui", failCheck2);
                        this.socket.removeListener("eval", caughtCheck);
                        this.socket.removeListener("player", failCheck3);
                        reject("We don't have a fishing rod equipped");
                    }
                }
                else if (typeof data == "object") {
                    if (data.response == "cooldown" && data.place == "fishing" && data.skill == "fishing") {
                        this.socket.removeListener("game_response", failCheck1);
                        this.socket.removeListener("ui", failCheck2);
                        this.socket.removeListener("eval", caughtCheck);
                        this.socket.removeListener("player", failCheck3);
                        reject(`Fishing is on cooldown (${data.ms}ms remaining)`);
                    }
                }
            };
            const failCheck2 = (data) => {
                if (data.type == "fishing_fail" && data.name == this.id) {
                    this.socket.removeListener("game_response", failCheck1);
                    this.socket.removeListener("ui", failCheck2);
                    this.socket.removeListener("eval", caughtCheck);
                    this.socket.removeListener("player", failCheck3);
                    reject("We failed to fish.");
                }
                else if (data.type == "fishing_none") {
                    this.socket.removeListener("game_response", failCheck1);
                    this.socket.removeListener("ui", failCheck2);
                    this.socket.removeListener("eval", caughtCheck);
                    this.socket.removeListener("player", failCheck3);
                    resolve();
                }
            };
            const failCheck3 = (data) => {
                if (!startedFishing && data.c.fishing) {
                    startedFishing = true;
                }
                else if (startedFishing && !data.c.fishing) {
                    this.socket.removeListener("game_response", failCheck1);
                    this.socket.removeListener("ui", failCheck2);
                    this.socket.removeListener("eval", caughtCheck);
                    this.socket.removeListener("player", failCheck3);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("game_response", failCheck1);
                this.socket.removeListener("ui", failCheck2);
                this.socket.removeListener("eval", caughtCheck);
                this.socket.removeListener("player", failCheck3);
                reject("fish timeout (20000ms)");
            }, 20000);
            this.socket.on("game_response", failCheck1);
            this.socket.on("eval", caughtCheck);
            this.socket.on("ui", failCheck2);
            this.socket.on("player", failCheck3);
        });
        this.socket.emit("skill", { name: "fishing" });
        return fished;
    }
    joinGiveaway(slot, id, rid) {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [joinGiveaway].");
        const merchant = this.players.get(id);
        if (!merchant || Tools_1.Tools.distance(this, merchant) > Constants_1.Constants.NPC_INTERACTION_DISTANCE)
            return Promise.reject(`${id} is too far away.`);
        if (!merchant.slots[slot]?.giveaway)
            return Promise.reject(`${id}'s slot ${slot} is not a giveaway.`);
        if (merchant.slots[slot]?.list.includes(id))
            return Promise.resolve();
        this.socket.emit("join_giveaway", { slot: slot, id: id, rid: rid });
    }
    listForSale(itemPos, tradeSlot, price, quantity = 1) {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [listForSale].");
        const itemInfo = this.items[itemPos];
        if (!itemInfo)
            return Promise.reject(`We do not have an item in slot ${itemPos}`);
        this.socket.emit("equip", {
            num: itemPos,
            q: quantity,
            slot: tradeSlot,
            price: price
        });
    }
    merchantCourage() {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [merchantCourage].");
        this.socket.emit("skill", { name: "mcourage" });
    }
    mine() {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [mine].");
        let startedMining = false;
        if (this.c.mining)
            startedMining = true;
        const mined = new Promise((resolve, reject) => {
            const caughtCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]mining['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.removeListener("game_response", failCheck1);
                    this.socket.removeListener("ui", failCheck2);
                    this.socket.removeListener("eval", caughtCheck);
                    this.socket.removeListener("player", failCheck3);
                    resolve();
                }
            };
            const failCheck1 = (data) => {
                if (typeof data == "string") {
                    if (data == "skill_cant_wtype") {
                        this.socket.removeListener("game_response", failCheck1);
                        this.socket.removeListener("ui", failCheck2);
                        this.socket.removeListener("eval", caughtCheck);
                        this.socket.removeListener("player", failCheck3);
                        reject("We don't have a pickaxe equipped");
                    }
                }
                else if (typeof data == "object") {
                    if (data.response == "cooldown" && data.place == "mining" && data.skill == "mining") {
                        this.socket.removeListener("game_response", failCheck1);
                        this.socket.removeListener("ui", failCheck2);
                        this.socket.removeListener("eval", caughtCheck);
                        this.socket.removeListener("player", failCheck3);
                        reject(`Mining is on cooldown (${data.ms}ms remaining)`);
                    }
                }
            };
            const failCheck2 = (data) => {
                if (data.type == "mining_fail" && data.name == this.id) {
                    this.socket.removeListener("game_response", failCheck1);
                    this.socket.removeListener("ui", failCheck2);
                    this.socket.removeListener("eval", caughtCheck);
                    this.socket.removeListener("player", failCheck3);
                    reject("We failed to mine.");
                }
                else if (data.type == "mining_none") {
                    this.socket.removeListener("game_response", failCheck1);
                    this.socket.removeListener("ui", failCheck2);
                    this.socket.removeListener("eval", caughtCheck);
                    this.socket.removeListener("player", failCheck3);
                    resolve();
                }
            };
            const failCheck3 = (data) => {
                if (!startedMining && data.c.mining) {
                    startedMining = true;
                }
                else if (startedMining && !data.c.mining) {
                    this.socket.removeListener("game_response", failCheck1);
                    this.socket.removeListener("ui", failCheck2);
                    this.socket.removeListener("eval", caughtCheck);
                    this.socket.removeListener("player", failCheck3);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("game_response", failCheck1);
                this.socket.removeListener("ui", failCheck2);
                this.socket.removeListener("eval", caughtCheck);
                this.socket.removeListener("player", failCheck3);
                reject("mine timeout (20000ms)");
            }, 20000);
            this.socket.on("game_response", failCheck1);
            this.socket.on("eval", caughtCheck);
            this.socket.on("ui", failCheck2);
            this.socket.on("player", failCheck3);
        });
        this.socket.emit("skill", { name: "mining" });
        return mined;
    }
    mluck(target) {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [mluck].");
        if (target !== this.id) {
            const player = this.players.get(target);
            if (!player)
                return Promise.reject(`Could not find ${target} to mluck.`);
            if (player.npc)
                return Promise.reject(`${target} is an NPC. You can't mluck NPCs.`);
            if (player.s.mluck && player.s.mluck.strong
                && ((player.owner && player.owner !== this.owner)
                    || (player.s.mluck.f !== this.id))) {
                return Promise.reject(`${target} has a strong mluck from ${player.s.mluck.f}.`);
            }
        }
        const mlucked = new Promise((resolve, reject) => {
            const mluckCheck = (data) => {
                for (const player of data.players) {
                    if (player.id == target
                        && player.s?.mluck?.f == this.id
                        && player.s?.mluck?.ms == this.G.conditions.mluck.duration) {
                        this.socket.removeListener("entities", mluckCheck);
                        this.socket.removeListener("game_response", failCheck);
                        this.socket.removeListener("player", selfMluckCheck);
                        resolve();
                    }
                }
            };
            const selfMluckCheck = (data) => {
                if (data.s?.mluck?.f == this.id
                    && data.s?.mluck?.ms == this.G.conditions.mluck.duration) {
                    this.socket.removeListener("entities", mluckCheck);
                    this.socket.removeListener("game_response", failCheck);
                    this.socket.removeListener("player", selfMluckCheck);
                    resolve();
                }
            };
            const failCheck = async (data) => {
                if (typeof data == "string") {
                    if (data == "skill_too_far") {
                        this.socket.removeListener("entities", mluckCheck);
                        this.socket.removeListener("game_response", failCheck);
                        this.socket.removeListener("player", selfMluckCheck);
                        await this.requestPlayerData().catch((e) => { console.error(e); });
                        reject(`We are too far from ${target} to mluck.`);
                    }
                    else if (data == "no_level") {
                        this.socket.removeListener("entities", mluckCheck);
                        this.socket.removeListener("game_response", failCheck);
                        this.socket.removeListener("player", selfMluckCheck);
                        reject("We aren't a high enough level to use mluck.");
                    }
                }
            };
            setTimeout(() => {
                this.socket.removeListener("entities", mluckCheck);
                this.socket.removeListener("game_response", failCheck);
                this.socket.removeListener("player", selfMluckCheck);
                reject(`mluck timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("game_response", failCheck);
            this.socket.on("player", selfMluckCheck);
            this.socket.on("entities", mluckCheck);
        });
        this.socket.emit("skill", { name: "mluck", id: target });
        return mlucked;
    }
    openMerchantStand() {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [openMerchantStand].");
        if (this.stand)
            return Promise.resolve();
        let stand = this.locateItem("stand0");
        if (stand === undefined) {
            stand = this.locateItem("computer");
            if (stand === undefined)
                return Promise.reject("Could not find merchant stand ('stand0') or computer in inventory.");
        }
        const opened = new Promise((resolve, reject) => {
            const checkStand = (data) => {
                if (data.stand) {
                    this.socket.removeListener("player", checkStand);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("player", checkStand);
                reject(`openMerchantStand timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("player", checkStand);
        });
        this.socket.emit("merchant", { num: stand });
        return opened;
    }
    massProduction() {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [massProduction].");
        const massProductioned = new Promise((resolve, reject) => {
            const productedCheck = (data) => {
                if (data.type == "massproduction" && data.name == this.id) {
                    this.socket.removeListener("ui", productedCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("ui", productedCheck);
                reject(`massProduction timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("ui", productedCheck);
        });
        this.socket.emit("skill", { name: "massproduction" });
        return massProductioned;
    }
}
exports.Merchant = Merchant;
//# sourceMappingURL=Merchant.js.map