"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Rogue = void 0;
const Constants_1 = require("./Constants");
const PingCompensatedCharacter_1 = require("./PingCompensatedCharacter");
class Rogue extends PingCompensatedCharacter_1.PingCompensatedCharacter {
    invis() {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [invis].");
        this.socket.emit("skill", { name: "invis" });
    }
    mentalBurst(target) {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [mentalBurst].");
        const bursted = new Promise((resolve, reject) => {
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]mentalburst['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.removeListener("eval", cooldownCheck);
                    this.socket.removeListener("death", deathCheck);
                    this.socket.removeListener("game_response", failCheck);
                    resolve();
                }
            };
            const deathCheck = (data) => {
                if (data.id == target) {
                    this.socket.removeListener("eval", cooldownCheck);
                    this.socket.removeListener("death", deathCheck);
                    this.socket.removeListener("game_response", failCheck);
                    reject(`Entity ${target} not found`);
                }
            };
            const failCheck = (data) => {
                if (typeof data == "object") {
                    if (data.response == "cooldown" && data.skill == "mentalburst") {
                        this.socket.removeListener("eval", cooldownCheck);
                        this.socket.removeListener("death", deathCheck);
                        this.socket.removeListener("game_response", failCheck);
                        reject(`mentalBurst on ${target} failed due to cooldown (ms: ${data.ms}).`);
                    }
                    else if (data.response == "too_far" && data.place == "mentalburst") {
                        this.socket.removeListener("eval", cooldownCheck);
                        this.socket.removeListener("death", deathCheck);
                        this.socket.removeListener("game_response", failCheck);
                        reject(`${target} is too far away to mentalBurst (dist: ${data.dist}).`);
                    }
                }
            };
            setTimeout(() => {
                this.socket.removeListener("eval", cooldownCheck);
                this.socket.removeListener("death", deathCheck);
                this.socket.removeListener("game_response", failCheck);
                reject(`mentalBurst timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("eval", cooldownCheck);
            this.socket.on("death", deathCheck);
            this.socket.on("game_response", failCheck);
        });
        this.socket.emit("skill", {
            id: target,
            name: "mentalburst"
        });
        return bursted;
    }
    poisonCoat() {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [poisonCoat].");
        const poisonCoated = new Promise((resolve, reject) => {
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]pcoat['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.removeListener("eval", cooldownCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("eval", cooldownCheck);
                reject(`poisoncoat timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", {
            name: "pcoat"
        });
        return poisonCoated;
    }
    quickPunch(target) {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [quickPunch].");
        const marked = new Promise((resolve, reject) => {
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]quickpunch['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.removeListener("eval", cooldownCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("eval", cooldownCheck);
                reject(`quickPunch timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", {
            id: target,
            name: "quickpunch"
        });
        return marked;
    }
    quickStab(target) {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [quickStab].");
        const stabbed = new Promise((resolve, reject) => {
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]quickstab['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.removeListener("eval", cooldownCheck);
                    this.socket.removeListener("death", deathCheck);
                    this.socket.removeListener("game_response", failCheck);
                    resolve();
                }
            };
            const failCheck = (data) => {
                if (typeof data == "object") {
                    if (data.response == "cooldown" && data.skill == "quickstab") {
                        this.socket.removeListener("eval", cooldownCheck);
                        this.socket.removeListener("death", deathCheck);
                        this.socket.removeListener("game_response", failCheck);
                        reject(`quickStab on ${target} failed due to cooldown (ms: ${data.ms}).`);
                    }
                    else if (data.response == "too_far" && data.place == "quickstab") {
                        this.socket.removeListener("eval", cooldownCheck);
                        this.socket.removeListener("death", deathCheck);
                        this.socket.removeListener("game_response", failCheck);
                        reject(`${target} is too far away to quickStab (dist: ${data.dist}).`);
                    }
                }
            };
            const deathCheck = (data) => {
                if (data.id == target) {
                    this.socket.removeListener("eval", cooldownCheck);
                    this.socket.removeListener("death", deathCheck);
                    this.socket.removeListener("game_response", failCheck);
                    reject(`Entity ${target} not found`);
                }
            };
            setTimeout(() => {
                this.socket.removeListener("eval", cooldownCheck);
                this.socket.removeListener("death", deathCheck);
                this.socket.removeListener("game_response", failCheck);
                reject(`quickStab timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("eval", cooldownCheck);
            this.socket.on("death", deathCheck);
            this.socket.on("game_response", failCheck);
        });
        this.socket.emit("skill", {
            id: target,
            name: "quickstab"
        });
        return stabbed;
    }
    rspeed(target) {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [rspeed].");
        const swifted = new Promise((resolve, reject) => {
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]rspeed['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.removeListener("eval", cooldownCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("eval", cooldownCheck);
                reject(`rspeed timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", {
            id: target,
            name: "rspeed",
        });
        return swifted;
    }
    shadowStrike(shadowstone = this.locateItem("shadowstone")) {
        if (!this.ready)
            return Promise.reject("We aren't ready yet [shadowStrike].");
        if (shadowstone === undefined)
            return Promise.reject("We need a shadowstone in order to shadowstrike.");
        const shadowStriked = new Promise((resolve, reject) => {
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]shadowstrike['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.removeListener("eval", cooldownCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("eval", cooldownCheck);
                reject(`shadowstrike timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", {
            name: "shadowstrike",
            num: shadowstone
        });
        return shadowStriked;
    }
}
exports.Rogue = Rogue;
//# sourceMappingURL=Rogue.js.map